unit main;

// Main form to test the Progress components.
// This form launches background threads that compute PI to
// an arbitrary number of decimal places. For each thread,
// it creates a progress component to show the thread's progress.
// A scroll box contains all the progress components, so you can
// easily create and examine many of them. A pop-up menu lets
// you control each thread--just right-click over the thread's
// progress meter and choose to Abort the thread, or if the thread
// is finished to display the result or delete the meter.
//
// Copyright  2002 Tempest Software, Inc.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, progress, StdCtrls, Menus, ExtCtrls;

type
  TMainForm = class(TForm)
    ScrollBox: TScrollBox;
    ComputePi: TButton;
    Label1: TLabel;
    Edit1: TEdit;
    NumDigits: TUpDown;
    PopupMenu: TPopupMenu;
    AbortThread: TMenuItem;
    ViewResult: TMenuItem;
    DeleteProgress: TMenuItem;
    ControlGroup: TRadioGroup;
    procedure ComputePiClick(Sender: TObject);
    procedure ViewResultClick(Sender: TObject);
    procedure AbortThreadClick(Sender: TObject);
    procedure PopupMenuPopup(Sender: TObject);
    procedure ThreadTerminate(Sender: TObject);
    procedure DeleteProgressClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ScrollBoxResize(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    NextTop: Integer;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses spigot, display;

{$R *.dfm}

procedure TMainForm.ComputePiClick(Sender: TObject);
var
  Progress: TThreadSafeProgressBar;
  PLabel: TThreadSafeProgressLabel;
  Control: TControl;
  Intf: IProgress;
  Thread: TThread;
begin
  // Start a background thread that computes pi, and display
  // a progress bar for the thread.
  if ControlGroup.ItemIndex = 0 then
  begin
    Progress := TThreadSafeProgressBar.Create(Self);
    Progress.Parent := ScrollBox;
    Progress.Top := NextTop;
    Progress.Width := ScrollBox.ClientWidth;
    Progress.PopupMenu := PopupMenu;
    Control := Progress;
    Intf := Progress;
  end
  else
  begin
    PLabel := TThreadSafeProgressLabel.Create(Self);
    PLabel.Parent := ScrollBox;
    PLabel.Top := NextTop;
    PLabel.Width := ScrollBox.ClientWidth;
    PLabel.PopupMenu := PopupMenu;
    PLabel.Suffix := ' digits';
    Control := PLabel;
    Intf := PLabel;
  end;

  // Stack all the progress bars on top of each other.
  Inc(NextTop, Control.Height);

  Thread := TSpigotThread.Create(NumDigits.Position, Intf);
  Thread.OnTerminate := ThreadTerminate;
  Control.Tag := Integer(Thread);
  Thread.Resume;
end;

procedure TMainForm.ViewResultClick(Sender: TObject);
var
  Thread: TSpigotThread;
begin
  // Popup a dialog box where the user can view the results
  // of the computation.
  Thread := TObject(((Sender as TMenuItem).GetParentMenu as TPopupMenu).PopupComponent.Tag) as TSpigotThread;
  Assert(Thread.Finished);
  DisplayPi(Thread.Pi)
end;

procedure TMainForm.AbortThreadClick(Sender: TObject);
var
  Thread: TSpigotThread;
begin
  // Kill the thread.
  Thread := TObject(((Sender as TMenuItem).GetParentMenu as TPopupMenu).PopupComponent.Tag) as TSpigotThread;
  Thread.Terminate;
end;

procedure TMainForm.PopupMenuPopup(Sender: TObject);
var
  Menu: TPopupMenu;
  Thread: TSpigotThread;
begin
  // Enable or disable menu items depending on the thread state.
  Menu := Sender as TPopupMenu;
  Thread := TObject(Menu.PopupComponent.Tag) as TSpigotThread;
  Menu.Items[0].Enabled := not Thread.Finished; // Abort
  Menu.Items[1].Enabled := Thread.Finished;     // View results
  Menu.Items[2].Enabled := Thread.Finished;     // Delete
end;

procedure TMainForm.ThreadTerminate(Sender: TObject);
var
  Thread: TSpigotThread;
begin
  // When the thread terminates, make sure the progress bar is pegged
  // to the maximum, so the user can see that it is finished.
  // This visual cue helps when the user aborts the thread.
  Thread := Sender as TSpigotThread;
  Thread.Progress.Value := Thread.Progress.Maximum;
end;

procedure TMainForm.DeleteProgressClick(Sender: TObject);
var
  Progress: TControl;
  Thread: TSpigotThread;
  I: Integer;
  Control: TControl;
begin
  // Delete a progress bar from the scroll box.
  Progress := ((Sender as TMenuItem).GetParentMenu as TPopupMenu).PopupComponent as TControl;
  Thread := TObject(Progress.Tag) as TSpigotThread;
  Assert(Thread.Finished);
  for I := 0 to ScrollBox.ControlCount-1 do
  begin
    // Slide all progress bars that are lower in the scroll box
    // up one notch, so there are no blank spaces.
    Control := ScrollBox.Controls[I];
    if Control.Top > Progress.Top then
      Control.Top := Control.Top - Progress.Height;
  end;
  // Adjust the position of the next progress bar.
  Dec(NextTop, Progress.Height);
  // Clean up.
  Progress.Tag := 0;
  Thread.Free;
  Progress.Free;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
var
  I: Integer;
  Progress: TControl;
  Thread: TSpigotThread;
begin
  // Kill all active threads and clean up.
  for I := 0 to ScrollBox.ControlCount-1 do
  begin
    Progress := ScrollBox.Controls[I];
    Thread := TObject(Progress.Tag) as TSpigotThread;
    Progress.Tag := 0;
    if Thread <> nil then
    begin
      Thread.Terminate;
      Thread.WaitFor;
      Thread.Free;
    end;
  end;
end;

procedure TMainForm.ScrollBoxResize(Sender: TObject);
var
  I: Integer;
begin
  // When the user resizes the form, the scroll box also resizes.
  // In that case, adjust the width of the progress bars to match
  // the scroll box. This looks much better than having a horizontal
  // scroll bar that serves no functional purpose.
  //
  // Note that the OnResize event handler is also called when the
  // scroll box adds or removes the vertical scroll bar.
  for I := 0 to ScrollBox.ControlCount-1 do
    ScrollBox.Controls[I].Width := ScrollBox.ClientWidth;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
resourcestring
  sPrompt = 'A background thread is still active. Are you sure you want to exit?';
var
  I: Integer;
begin
  // If any threads are still active, prompt the user before closing.
  for I := 0 to ScrollBox.ControlCount-1 do
    if not (TObject(ScrollBox.Controls[I].Tag) as TSpigotThread).Finished then
    begin
      CanClose := MessageDlg(sPrompt, mtWarning, [mbYes, mbNo], 0) = mrYes;
      Exit;
    end;
end;

end.
