unit progress;

interface

// Display the progress of a thread.
// Any component that can display progress in a thread-safe
// manner can implement the IProgress interface. Only GetValue
// and SetValue are thread safe--SetLimits must be called
// in a protected manner, e.g., from the TThread.Create constructor.
//
// The TThreadSafeProgressBar is an example of a TProgressBar
// component that implement IProgress. It achieves thread safety
// by saving the value and using a timer to update the progress.
// This design avoids any need for the clumsy Syncrhonize method
// and ensure timely updating of the progress bar (assuming the
// thread is actually making progress).
//
// TThreadSafeProgressLabel is another example. The text displays
// the numerical progress with a prefix and suffix. For example,
// the limits might be 0 to 100, with a suffix of "%".
//
// Copyright  2002 Tempest Software, Inc.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

uses Windows, SysUtils, Classes, Controls, StdCtrls, ExtCtrls, ComCtrls;

type
  IProgress = interface
  ['{F43E6C6D-5592-4853-9FDB-32D7DB0CF74C}']
    function GetMaximum: Integer;
    function GetMinimum: Integer;
    function GetValue: Integer;
    procedure SetLimits(Min, Max: Integer);
    procedure SetValue(NewValue: Integer);

    property Maximum: Integer read GetMaximum;
    property Minimum: Integer read GetMinimum;
    property Value: Integer read GetValue write SetValue;
  end;

  // Thread-safe progress bar. The progress can be
  // set from any thread by using the IProgress interface.
  TThreadSafeProgressBar = class(TProgressBar, IProgress)
  private
    fTimer: TTimer;
    fUpdateNeeded: Boolean;
    fValue: Integer;
    procedure OnTimer(Sender: TObject);
    function GetInterval: Cardinal;
    procedure SetInterval(const Value: Cardinal);
  protected
    property Timer: TTimer read fTimer;
    property UpdateNeeded: Boolean read fUpdateNeeded;
  public
    constructor Create(Owner: TComponent); override;
    function GetMaximum: Integer;
    function GetMinimum: Integer;
    function GetValue: Integer;
    procedure SetLimits(Min: Integer; Max: Integer);
    procedure SetValue(NewValue: Integer);
  published
    property Interval: Cardinal read GetInterval write SetInterval;
    property Position: Integer read GetValue write SetValue;
  end;

  TThreadSafeProgressLabel = class(TCustomLabel, IProgress)
  private
    fMaximum: Integer;
    fMinimum: Integer;
    fPrefix: string;
    fSuffix: string;
    fTimer: TTimer;
    fUpdateNeeded: Boolean;
    fValue: Integer;
    fPercent: Boolean;
    function GetMaximum: Integer;
    function GetMinimum: Integer;
    function GetValue: Integer;
    procedure SetLimits(Min, Max: Integer);
    procedure SetValue(NewValue: Integer);
    procedure SetMaximum(const Value: Integer);
    procedure SetMinimum(const Value: Integer);
    procedure SetPrefix(const Value: string);
    procedure SetSuffix(const Value: string);

    procedure OnTimer(Sender: TObject);
    function GetInterval: Cardinal;
    procedure SetInterval(const Value: Cardinal);
    procedure SetPercent(const Value: Boolean);
  protected
    procedure SetCaption;
    property Timer: TTimer read fTimer;
    property UpdateNeeded: Boolean read fUpdateNeeded;
  public
    constructor Create(Owner: TComponent); override;
  published
    property Interval: Cardinal read GetInterval write SetInterval default 250;
    property Maximum: Integer read fMaximum write SetMaximum default 100;
    property Minimum: Integer read fMinimum write SetMinimum default 0;
    property Percent: Boolean read fPercent write SetPercent;
    property Prefix: string read fPrefix write SetPrefix;
    property Suffix: string read fSuffix write SetSuffix;
    property Value: Integer read fValue write SetValue default 0;

    property Align;
    property Alignment;
    property Anchors;
    property AutoSize;
    property BiDiMode;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FocusControl;
    property Font;
    property ParentBiDiMode;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property Transparent;
    property Layout;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnStartDock;
    property OnStartDrag;
  end;

implementation

uses Consts;

{ TThreadSafeProgressBar }

constructor TThreadSafeProgressBar.Create(Owner: TComponent);
begin
  fTimer := TTimer.Create(Self);
  inherited;
  Timer.Interval := 250;
  Timer.OnTimer := OnTimer;
end;

function TThreadSafeProgressBar.GetInterval: Cardinal;
begin
  Result := Timer.Interval
end;

function TThreadSafeProgressBar.GetMaximum: Integer;
begin
  Result := Max
end;

function TThreadSafeProgressBar.GetMinimum: Integer;
begin
  Result := Min
end;

function TThreadSafeProgressBar.GetValue: Integer;
begin
  Result := fValue
end;

procedure TThreadSafeProgressBar.OnTimer(Sender: TObject);
begin
  if UpdateNeeded then
    inherited Position := Position;
end;

procedure TThreadSafeProgressBar.SetInterval(const Value: Cardinal);
begin
  Timer.Interval := Value
end;

procedure TThreadSafeProgressBar.SetLimits(Min, Max: Integer);
begin
  if Max < Min then
    raise EInvalidOperation.CreateFmt(SPropertyOutOfRange, [Classname]);
  Self.Min := Min;
  Self.Max := Max;
  if csDesigning in ComponentState then
  begin
    if Position < Min then
      Position := Min
    else if Position > Max then
      Position := Max;
  end;
end;

procedure TThreadSafeProgressBar.SetValue(NewValue: Integer);
begin
  if csDesigning in ComponentState then
  begin
    if (NewValue < Min) or (NewValue > Max) then
      raise EInvalidOperation.CreateFmt(SOutOfRange, [Min, Max]);
  end;

  fUpdateNeeded := NewValue <> Position;
  fValue := NewValue;
end;


{ TThreadSafeProgressLabel }

constructor TThreadSafeProgressLabel.Create(Owner: TComponent);
begin
  fTimer := TTimer.Create(Self);
  inherited;
  Timer.Interval := 250;
  Timer.OnTimer := OnTimer;
  fMaximum := 100;
  Caption := '0';
end;

function TThreadSafeProgressLabel.GetInterval: Cardinal;
begin
  Result := Timer.Interval
end;

function TThreadSafeProgressLabel.GetMaximum: Integer;
begin
  Result := fMaximum
end;

function TThreadSafeProgressLabel.GetMinimum: Integer;
begin
  Result := fMinimum
end;

function TThreadSafeProgressLabel.GetValue: Integer;
begin
  Result := fValue
end;

procedure TThreadSafeProgressLabel.OnTimer(Sender: TObject);
begin
  SetCaption;
end;

procedure TThreadSafeProgressLabel.SetCaption;
begin
  if Percent then
    Caption := Prefix + IntToStr(MulDiv(Value, 100, Maximum - Minimum)) + Suffix
  else
    Caption := Prefix + IntToStr(Value) + Suffix;
end;

procedure TThreadSafeProgressLabel.SetInterval(const Value: Cardinal);
begin
  Timer.Interval := Value;
end;

procedure TThreadSafeProgressLabel.SetLimits(Min, Max: Integer);
begin
  if Max < Min then
    raise EInvalidOperation.CreateFmt(SPropertyOutOfRange, [Classname]);
  fMinimum := Min;
  fMaximum := Max;
  if csDesigning in ComponentState then
  begin
    if Value < Minimum then
      Value := Minimum
    else if Value > Maximum then
      Value := Maximum;
    SetCaption;
  end;
end;

procedure TThreadSafeProgressLabel.SetMaximum(const Value: Integer);
begin
  SetLimits(Minimum, Value)
end;

procedure TThreadSafeProgressLabel.SetMinimum(const Value: Integer);
begin
  SetLimits(Value, Maximum)
end;

procedure TThreadSafeProgressLabel.SetPercent(const Value: Boolean);
begin
  fPercent := Value;
  if csDesigning in ComponentState then
  begin
    SetCaption;
    if Suffix = '' then
      Suffix := '%';
  end;
end;

procedure TThreadSafeProgressLabel.SetPrefix(const Value: string);
begin
  fPrefix := Value;
  if csDesigning in ComponentState then
    SetCaption;
end;

procedure TThreadSafeProgressLabel.SetSuffix(const Value: string);
begin
  fSuffix := Value;
  if csDesigning in ComponentState then
    SetCaption;
end;

procedure TThreadSafeProgressLabel.SetValue(NewValue: Integer);
begin
  if csDesigning in ComponentState then
  begin
    if (fValue < Minimum) or (fValue > Maximum) then
      raise Exception.CreateFmt(SOutOfRange, [Minimum, Maximum]);
    fValue := NewValue;
    SetCaption;
  end
  else
    fValue := NewValue;
end;

end.
