unit spigot;

interface

// Compute digits of pi using the Rabinowitz and Wagon "spigot" algorithm.
// (Rabinowitz, S. and Wagon, S., A spigot algorithm for the digits of pi,
// American Mathematical Monthly, v. 102, n. 3, March 1995, pp. 195-203.)
// This algorithm is slow, but it is easy to code and does not use an
// enormous amount of memory.
//
// Copyright  2002 Tempest Software, Inc.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


uses
  Classes, progress;

type
  TSpigotThread = class(TThread)
  private
    fFinished: Boolean;
    fNumDigits: Integer;
    fPi: string;
    PiLength: Integer;
    fProgress: IProgress;
  protected
    procedure Execute; override;
  public
    constructor Create(NumDigits: Integer; const Progress: IProgress);
    property Finished: Boolean read fFinished;
    property NumDigits: Integer read fNumDigits;
    property Pi: string read fPi;
    property Progress: IProgress read fProgress;
  end;

implementation

{ TSpigotThread }

constructor TSpigotThread.Create(NumDigits: Integer; const Progress: IProgress);
begin
  inherited Create(True);
  fNumDigits := NumDigits;
  fProgress := Progress;
  Progress.SetLimits(0, NumDigits);
end;

{$R-,Q-} // For maximum performance
procedure TSpigotThread.Execute;
var
  A: array of LongInt;
  I, J, K, P, Q, X, Nines, Predigit: Integer;
begin
  SetLength(A, 10*NumDigits div 3);
  SetLength(fPi, NumDigits+1);
  PiLength := 1;
  for I := Low(A) to High(A) do
    A[I] := 2;
  Nines := 0;
  Predigit := 0;
  for J := 0 to NumDigits-1 do
  begin
    Q := 0;
    P := 2 * High(A) + 1;
    for I := High(A) downto Low(A) do
    begin
      X := 10*A[I] + Q*(I+1);
      A[I] := X mod P;
      Q := X div P;
      P := P - 2;
    end;
    A[Low(A)] := Q mod 10;
    Q := Q div 10;
    if Q = 9 then
      Inc(Nines)
    else if Q = 10 then
    begin
      fPi[PiLength] := Chr(Predigit + 1 + Ord('0'));
      for K := 1 to Nines do
        fPi[PiLength+K] := '0';
      PiLength := PiLength + Nines + 1;
      Predigit := 0;
      Nines := 0;
    end
    else
    begin
      fPi[PiLength] := Chr(Predigit + Ord('0'));
      Predigit := Q;
      for K := 1 to Nines do
        fPi[PiLength+K] := '9';
      PiLength := PiLength + Nines + 1;
      Nines := 0;
    end;
    Progress.Value := PiLength;
    if Terminated then
    begin
      SetLength(fPi, PiLength);
      fFinished := True;
      Exit;
    end;
  end;
  fPi[PiLength] := Chr(Predigit + Ord('0'));
  fFinished := True;
end;

end.
